@@ -8,7 +8,7 @@ our $VERSION = '4.86';
 # We export some stuff
 require Exporter;
 our @ISA = qw( Exporter );
-our %EXPORT_TAGS = ( 'ALL' => [ qw(u_irc l_irc parse_mode_line parse_ban_mask) ] );
+our %EXPORT_TAGS = ( 'ALL' => [ qw(u_irc l_irc parse_mode_line parse_ban_mask matches_mask parse_user) ] );
 Exporter::export_ok_tags( 'ALL' );
 
 sub u_irc {
@@ -90,6 +90,40 @@ sub parse_ban_mask {
   return $ban[0] . '!' . $ban[1] . '@' . $ban[2];
 }
 
+sub matches_mask_array {
+  my ($masks,$matches,$mapping) = @_;
+  return unless $masks and $matches;
+  return unless ref $masks eq 'ARRAY';
+  return unless ref $matches eq 'ARRAY';
+  my $ref = { };
+  foreach my $mask ( @{ $masks } ) {
+        foreach my $match ( @{ $matches } ) {
+           push @{ $ref->{ $mask } }, $match if matches_mask( $mask, $match, $mapping );
+        }
+  }
+  return $ref;
+}
+
+sub matches_mask {
+  my ($mask,$match,$mapping) = @_;
+  return unless $mask and $match;
+  $mask = parse_ban_mask( $mask );
+  $mask =~ s/\x2A+/\x2A/g;
+  my $umask = quotemeta u_irc( $mask, $mapping );
+  $umask =~ s/\\\*/[\x01-\xFF]{0,}/g;
+  $umask =~ s/\\\?/[\x01-\xFF]{1,1}/g;
+  $match = u_irc $match, $mapping;
+  return 1 if $match =~ /^$umask$/;
+  return 0;
+}
+
+sub parse_user {
+  my $user = shift || return;
+  my ($n,$u,$h) = split /[!@]/, $user;
+  return ($n,$u,$h) if wantarray();
+  return $n;
+}
+
 1;
 __END__
 
@@ -115,6 +149,13 @@ POE::Component::IRC::Common - provides a set of common functions for the L<POE::
   my $banmask = 'stalin*';
   $full_banmask = parse_ban_mask( $banmask );
 
+  if ( matches_mask( $full_banmask, 'stalin!joe@kremlin.ru' ) ) {
+	print "EEK!";
+  }
+
+  my $nick = parse_user( 'stalin!joe@kremlin.ru' );
+  my ($nick,$user,$host) = parse_user( 'stalin!joe@kremlin.ru' );
+
 =head1 DESCRIPTION
 
 POE::Component::IRC::Common provides a set of common functions for the L<POE::Component::IRC> suite. There are included functions for uppercase and lowercase nicknames/channelnames and for parsing mode lines and ban masks.
@@ -158,6 +199,14 @@ Example:
 
    $fullbanmask will be 'stalin*!*@*';
 
+=item matches_mask
+
+Takes two parameters, a string representing an IRC mask ( it'll be processed with parse_ban_mask() to ensure that it is normalised ) and something to match against the IRC mask, such as a nick!user@hostname string. Returns 1 if they match, 0 otherwise. Returns undef if parameters are missing. Optionally, one may pass the casemapping ( see u_irc() ), as this function ises u_irc() internally.
+
+=item parse_user
+
+Takes one parameter, a string representing a user in the form nick!user@hostname. In a scalar context it returns just the nickname. In a list context it returns a list consisting of the nick, user and hostname, respectively.
+
 =back
 
 =head1 AUTHOR
